@@ -14,9 +14,15 @@ class ExcelWrapper:
     def get_macrosheets(self):
         pass
 
+    def get_worksheets(self):
+        pass
+
     def get_cell_info(self, sheet_name, col, row, info_type_id):
         pass
 
+    def get_workbook_name(self):
+        pass
+
 
 class XlApplicationInternational(Enum):
     # https://docs.microsoft.com/en-us/office/vba/api/excel.xlapplicationinternational
